#include "util.h"

void SocketTimeoutHandler(int sig) {
	cout << "Socket timeout after " << SOCKET_TIMEOUT << " seconds" << endl;	
	exit(STATE_CRITICAL);
}

int ChkPasswd(char *passwd, int &passwdLen) {
	int fd;
	
	fd = open(PASSWDFILE, O_RDONLY);

	if( fd == -1 ) {
		if( errno == ENOENT )
			cout << "Error: Password file missing" << endl;
		else
			cout << "Error: " << strerror(errno) << endl;
			
		return STATE_CRITICAL;
	}
			
	// File is open, read password
	passwdLen = read(fd, passwd, MAX_PASSWD_LEN);	
	if(passwdLen == -1) 
	{
	   cout << strerror(errno) << endl;
	   close(fd);
	   return STATE_CRITICAL;
	}

 	close(fd);		
			
	passwd[passwdLen-1] = '\0';		

	return STATE_OK;
}

bool Send(int &sockfd, char *buffer)
{	
	int  n;
	int  index=0;
	int  nBytesToSend = strlen(buffer)+20;
	char* tmpBuf = new char[nBytesToSend];
	char obuf[BUF_SIZE+1];
	char in;
	
	buffer[strlen(buffer)]='\0';	
	n  = sprintf(tmpBuf, "nbytes=%d;\0", nBytesToSend);		
	strcat(tmpBuf, buffer);	
	nBytesToSend = strlen(tmpBuf);
	
	while(nBytesToSend>0)
	{		
		strncpy(obuf, &tmpBuf[index*BUF_SIZE], BUF_SIZE);			
		obuf[BUF_SIZE]='\0';				
		index++;		
		
		n = send( sockfd, obuf, strlen(obuf), 0);
		nBytesToSend -= n;		
						
		if(nBytesToSend<=0)
		{			
			delete tmpBuf;
			return 1;
		}
	 
		if(n<0)
		{    	
	    	cout << strerror(errno) << endl;
	    	delete tmpBuf;  
			return 0;
		}		
		
		signal(SIGALRM, SocketTimeoutHandler);        
		alarm(SOCKET_TIMEOUT);     	     	
    
		n = recv( sockfd, &in, BUF_SIZE, 0);
		alarm(0);
		
		if( n < 0 ) 
	    {    	
	    	cout << strerror(errno) << endl;
	    	delete tmpBuf;
	    	return 0;
	    }	    
	}	
	
	delete tmpBuf;
	return 1;
}

char* Receive( int &sockfd )			   
{
	int 	index = 0; 
	int		nBytesRecv=0;
    int 	n;        
    char  	ch='1';  
    char	*ibuf = new char[BUF_SIZE+1];
    char 	*buffer;    
    int		nBytes;	
		
	signal(SIGALRM, SocketTimeoutHandler);        
	alarm(SOCKET_TIMEOUT);	    
     	     	
    memset(ibuf,0, BUF_SIZE);
	n = recv( sockfd, ibuf, BUF_SIZE, 0);
	ibuf[n]='\0';
	nBytesRecv = n;		
	
	// reset the alarm
	alarm(0);	
    
    if( n < 0 ) 
    {    	
    	cout << strerror(errno) << endl;    		
    	return 0;
    }	
	
	if( strncmp(ibuf, "nbytes=", 7) == 0)
	{
    	nBytes = atoi(&ibuf[7]);
    	
    	if( nBytes < BUF_SIZE )
    	{
    		ibuf[strlen(ibuf)]='\0';    		
    		return ibuf;    	
    	}
    		
    	buffer = new char[nBytes];    	
    	strncpy(buffer, ibuf, nBytes);      	  		
    }
    else    	
    	return ibuf;    
    
    while(1)
    {    	
    	// we are ready to receive next message
    	send(sockfd, ibuf, 1, 0);
    	
    	signal(SIGALRM, SocketTimeoutHandler);        
		alarm(SOCKET_TIMEOUT);		
		
		// wait for next message
    	memset(ibuf,0, BUF_SIZE);
    	//cout << "Waiting..." << endl;
		n = recv( sockfd, ibuf, BUF_SIZE, 0);
		ibuf[n]='\0';
		nBytesRecv += n;
		strncat(buffer, ibuf, n);		
		
		alarm(0);    
	    if( n < 0 ) 
	    {    	
	    	cout << strerror(errno) << endl;    		
	    	return 0;
	    }   
	    
	    while( index < nBytesRecv && nBytesRecv <= nBytes)
	    {
	    	if( strncmp(&buffer[index], ";;", 2) == 0)
	    	{
	    		buffer[index+2]='\0';    			    	
	    		return buffer;
	    	}
	    	
	    	index++;
	    	
	    	if(nBytes < index)
	    	{
	    		cout << "Error in Receive" << endl;
	    		return 0;
	    	}
	    }	    
	}
	
	return buffer;
}