/*******************************************************************
 *
 * check_ntload plugin
 * Copyright (c) 2000 Jan Christian Kaldestad 
 *
 *
 * Description:
 * 	 Checks the processor load average on NT servers.  
 *	 Also finds the process with highest cpu usage.
 *
 ******************************************************************/
 
#include <iostream.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <sys/errno.h>
#include <signal.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>

#define SA					struct sockaddr

#define STATE_OK			 0
#define STATE_WARNING		 1
#define STATE_CRITICAL		 2
#define STATE_UNKNOWN		-1

#define MAX_INTERVAL		15
#define BUF_SIZE			500
#define SOCKET_TIMEOUT		10	// seconds

#define PASSWDFILE			"../etc/ntpasswd"
#define MAX_PASSWD_LEN		19 

// message type
#define CPULOAD_QUERY		10

void SocketTimeoutHandler(int);
int CheckCmdLine(int argc, char **argv, int &port, int &interval1, int &wload1, int &cload1, 
				 int &interval2, int &wload2, int &cload2);
int AnalayzeMessage(char *ibuf, int &cpuLoad1, int &cpuLoad2, int &prosLoad, char *prosName);
int ChkPasswd(char *passwd,int &passwdLen);
int CheckLoad(int cpuLoad2, int wload2, int cload2);
void PrintMsg(int state, int interval1, int cpuLoad1, int interval2, int cpuLoad2, char* prosName, int prosLoad);

int main(int argc, char **argv)
{
    int sockfd, n, cpuLoad1, cpuLoad2, prosLoad;
    char ibuf[BUF_SIZE], obuf[BUF_SIZE];
    char prosName[300];
    char passwd[MAX_PASSWD_LEN];
    int  passwdLen;
    struct sockaddr_in servaddr;
    int servPort;
    
    int err;
    int interval1, interval2, wload1, wload2, cload1, cload2;
   
    // Get password from passordfile
   if( ChkPasswd(passwd, passwdLen) == STATE_CRITICAL )    	
    	return STATE_CRITICAL;
   
    err = CheckCmdLine(argc, argv, servPort, interval1, wload1, cload1, interval2, wload2, cload2);
    
    if( err == STATE_UNKNOWN )
    	return STATE_UNKNOWN;   
           
    memset(&servaddr, 0, sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_port = htons(servPort);
    inet_pton(AF_INET, argv[1], &servaddr.sin_addr);
    
   // Initialize server-adress
	memset(&servaddr, 0, sizeof(servaddr));
	servaddr.sin_family		 = AF_INET;
	servaddr.sin_port		 = htons(servPort);
		
	err = inet_aton(argv[1], &servaddr.sin_addr);
	
	if(err == 0) {
		cout << "Invalid IP adress" << endl;
		return STATE_UNKNOWN;
	}	

    sockfd = socket(AF_INET, SOCK_DGRAM, 0);
    
    
    int j;
   
    // build buffer to send  
    j  = sprintf( obuf, "id=%d;", CPULOAD_QUERY);
    j += sprintf( obuf+j, "key=%s\n;", passwd);
    j += sprintf( obuf+j, "i1=%d;", interval1); 
    j += sprintf( obuf+j, "i2=%d\0", interval2);  
            
    sendto( sockfd, obuf, sizeof(obuf), 0, (SA *) &servaddr, sizeof(servaddr) );    
     
    signal(SIGALRM, SocketTimeoutHandler);
        
    alarm(SOCKET_TIMEOUT);
    n = recvfrom( sockfd, ibuf, sizeof(ibuf), 0, NULL, NULL );
    
    // reset alarm
    alarm(0);	
    
    if( n < 0 ) {    	
    	cout << strerror(errno) << endl;    		
    	return STATE_UNKNOWN;
    }	   	
	
	// get values from received buffer
	err = AnalayzeMessage(ibuf, cpuLoad1, cpuLoad2, prosLoad, prosName);
		
	if(err == STATE_UNKNOWN)
		return STATE_UNKNOWN;
		
	int state, ret1, ret2;	
	
	ret1 = CheckLoad(cpuLoad1, wload1, cload1);
	ret2 = CheckLoad(cpuLoad2, wload2, cload2);
		
	if(ret1>ret2)
		state=ret1;
	else 
		state=ret2;
		
	// print output-text
	PrintMsg(state, interval1, cpuLoad1, interval2, cpuLoad2, prosName, prosLoad);
	
	return state;	        
}


int CheckCmdLine(int argc, char **argv, int &port, int &interval1, int &wload1, int &cload1,
				 int &interval2, int &wload2, int &cload2) {
	if (argc < 9) {
		cout << "Incorrect arguments supplied\n\n";		
    	cout << "Usage: " << argv[0] << " <IPaddress> <port> <interval1> <wload1> <cload1> <interval2> <wload2> <cload2>\n\n";
    	cout << "Note:\n";
    	cout << " <interval>  = time interval in minutes used to measure the cpu load average," << endl;
    	cout << "               must be a number between 0 and 15 minutes" << endl;
    	cout << " <wload>     = cpu load limit to result in a WARNING message," << endl;
    	cout << "               must be a number between 0 and 100" << endl;
    	cout << " <cload>     = cpu load limit to result in a CRITICAL message," << endl;
    	cout << "               must be a number between 0 and 100" << endl;
    	
    	return STATE_UNKNOWN;
    }    
    
    port = atoi(argv[2]);
    if(port<1025 | port>65535) {
    	cout << "Invalid port. Must be a number between 1025 and 65535" << endl;
    	return STATE_UNKNOWN;
    }    
    
    interval1 = atoi(argv[3]);
    wload1	  = atoi(argv[4]);
    cload1	  = atoi(argv[5]);
    interval2 = atoi(argv[6]);
    wload2	  = atoi(argv[7]);
    cload2	  = atoi(argv[8]);
        
    if( (interval1<0) || (interval1>MAX_INTERVAL)) {
    	cout << "Usage: " << argv[0] << " <IPaddress> <port> <interval1> <wload1> <cload1> <interval2> <wload2> <cload2>\n\n";
    	cout << " <interval1> must be a number between 0 and 15 minutes" << endl;
    	return STATE_UNKNOWN;
    }
    
    if( (wload1<0) || (wload1>100) || (cload1<0) || (cload1>100) ) {
    	cout << "Usage: " << argv[0] << " <IPaddress> <port> <interval1> <wload1> <cload1> <interval2> <wload2> <cload2>\n\n";
    	cout << " <wload1> and <cload1> must be a number between 0 and 100" << endl;
    	return STATE_UNKNOWN;
      }
       
    if(wload1>cload1) {
    	cout << "Inconsitstence in parameters: \"warning load\" greater than \"critical load\"" << endl;
    	return STATE_UNKNOWN;
    }
    
    if( (interval2<0) || (interval2>MAX_INTERVAL)) {
    	cout << "Usage: " << argv[0] << " <IPaddress> <port> <interval1> <wload1> <cload1> <interval2> <wload2> <cload2>\n\n";
    	cout << " <interval2> must be a number between 0 and 15 minutes" << endl;
    	return STATE_UNKNOWN;
    }
    
    if( (wload2<0) || (wload2>100) || (cload2<0) || (cload2>100) ) {
    	cout << "Usage: " << argv[0] << " <IPaddress> <port> <interval1> <wload1> <cload1> <interval2> <wload2> <cload2>\n\n";
    	cout << " <wload2> and <cload2> must be a number between 0 and 100" << endl;
    	return STATE_UNKNOWN;
      }
       
    if(wload2>cload2) {
    	cout << "Inconsitstence in parameters: \"warning load\" greater than \"critical load\"" << endl;
    	return STATE_UNKNOWN;
    }
    
	return 0;
}

void SocketTimeoutHandler(int sig) {
	cout << "Socket timeout after " << SOCKET_TIMEOUT << " seconds" << endl;	
	exit(STATE_CRITICAL);
}

int AnalayzeMessage(char *ibuf, int &cpuLoad1, int &cpuLoad2, int &prosLoad, char *prosName) {
	
	int index=0;
	int index2=0;		
	
	// message format: id=10;l1=X;l2=Y;pname=N;pload=Z;
	if(atoi(&ibuf[3]) != CPULOAD_QUERY ) 
	{
		cout << "Error in server reply" << endl;
		return STATE_UNKNOWN;
	}	
	
	while( strncmp(&ibuf[index], "l1=", 3) != 0 && index < strlen(ibuf))
		index++;
		
	index+=3;
	cpuLoad1 = atoi(&ibuf[index]);	
	
	while( strncmp(&ibuf[index], "l2=", 3) != 0 && index < strlen(ibuf))
		index++;
		
	index+=3;
	cpuLoad2 = atoi(&ibuf[index]);	
		
	while( strncmp(&ibuf[index], "pname=", 6) != 0 && index < strlen(ibuf))
		index++;
		
	index +=6;
	index2 = index;
	
	while( ibuf[index2] != ';' && index2 < strlen(ibuf))
		index2++;
	
	strncpy(prosName, &ibuf[index], index2-index);
	prosName[index2-index]='\0';
	
	index = index2;	

	while( strncmp(&ibuf[index], ";pload=", 7) != 0 && index < strlen(ibuf))
		index++;
	
	index+=7;
	prosLoad = atoi(&ibuf[index]);	
	
	return STATE_OK;
}

int ChkPasswd(char *passwd, int &passwdLen) {
	int fd;
	
	fd = open(PASSWDFILE, O_RDONLY);

	if( fd == -1 ) {
		if( errno == ENOENT )
			cout << "Error: Password file missing" << endl;
		else
			cout << "Error: " << strerror(errno) << endl;
			
		return STATE_CRITICAL;
	}			
	
	passwdLen = read(fd, passwd, MAX_PASSWD_LEN);	
	if(passwdLen == -1) 
	{
	   cout << strerror(errno) << endl;
	   close(fd);
	   exit(1);
	}

 	close(fd);		
			
	passwd[passwdLen-1] = '\0';	

	return STATE_OK;
}

int CheckLoad(int cpuLoad, int wload, int cload) 
{
	if (cpuLoad < wload)		
		return STATE_OK;		
		
	if (cpuLoad < cload)
		return STATE_WARNING;
			
	return STATE_CRITICAL;	
}

void PrintMsg(int state, int interval1, int cpuLoad1, int interval2, int cpuLoad2, char* prosName, int prosLoad)
{
	switch(state)
	{
		case STATE_OK:			
			cout << "CPU Load OK (" << interval1 << "m: " << cpuLoad1 << "%) " 
				 << "(" << interval2 << "m: " << cpuLoad2 << "%)" << endl;
			break;
		default:
			if(prosLoad>0) 
			{				
				cout << "CPU Load HIGH (" << interval1 << "m: " << cpuLoad1 << "%) " 
			 		 << "(" << interval2 << "m: " << cpuLoad2 << "%) Process " << prosName
			 	 	 << " using " << prosLoad << "%" << endl; 
			}
			else 
			{
				cout << "CPU Load HIGH (" << interval1 << "m: " << cpuLoad1 << "%) " 
			 		 << "(" << interval2 << "m: " << cpuLoad2 << "%) Unknown process" << endl; 
			}				
	}
	
}