#include "checkService.h"

DWORD CheckService(char* servName, DWORD &stat)
{
	SC_HANDLE			schService;
    SC_HANDLE			schSCManager;
	SERVICE_STATUS		ssStatus;
	DWORD				dwRet;
	DWORD				bufSize=0;
	char				*lpServiceName=0;

	stat = -1;

	// Open a handle to the SC Manager database. 
	schSCManager = OpenSCManager( NULL,				// machine (NULL == local)
								  NULL,				// database (NULL == default)
								  GENERIC_READ );	// access required
		
	if ( schSCManager ) 
	{			
		GetServiceKeyName(	schSCManager,	// handle to a service control manager database
							servName,		// the service's display name
							lpServiceName,	// buffer to receive the service name
							&bufSize );		// size of service name buffer and service name

		lpServiceName = new char[bufSize+1];
		
		GetServiceKeyName(	schSCManager,	
							servName,		
							lpServiceName,	
							&bufSize );
		
		// Open a handle to the service.
		schService = OpenService( schSCManager,			 // SCM database 
								  lpServiceName,		 // service name
								  SERVICE_INTERROGATE ); // Requests the service to update  
														 // immediately its current status 
														 // information to the service control
														 // manager
		delete lpServiceName;
				
		if (schService == NULL)  
		{
			dwRet = GetLastError();
			CloseServiceHandle(schSCManager);			
			return dwRet;						
		}
		
		// Send a interrogate control code to the service.
		if( !ControlService( schService,					// handle to service	 
							 SERVICE_CONTROL_INTERROGATE,   // control code
							 &ssStatus ))					// pointer to service status structure
		{
			dwRet = GetLastError();
			CloseServiceHandle(schSCManager);
			CloseServiceHandle(schService);			
			return dwRet;			
		}
	}
	else
		return GetLastError();
	
	stat = ssStatus.dwCurrentState;

	CloseServiceHandle(schSCManager);
	CloseServiceHandle(schService);
	
	return ERROR_SUCCESS;
}

BOOL CheckServices(char *ibuf, char* obuf, DWORD &nBytes )
{	
	DWORD	index=0;
	DWORD	obufIndex=100;
	DWORD	i2=0;
	char	*servName=0;
	int		nameLen;
	DWORD	stat;
	DWORD	err;
	bool	ok=true;
	DWORD	bytesNeeded;
	
	bytesNeeded = GetBytesNeededForServiceCheckBuffer(ibuf);		


	if(nBytes < bytesNeeded)
	{
		nBytes =  bytesNeeded+100;		
		return 0;
	}	
	
	sprintf( obuf, "nbytes=%d;", bytesNeeded+100);

	while( strncmp(&ibuf[index], "key=", 4) != 0 && index < strlen(ibuf))
		index++;

	while(index < strlen(ibuf))
	{
		while( ibuf[index]!=';' && index < strlen(ibuf))
			index++;

		if(ibuf[index+1]==';')
			break;		

		i2 = ++index;

		while( ibuf[i2]!=';' && i2 < strlen(ibuf))
			i2++;

		nameLen = i2-index+1;

		servName = new char[nameLen];
		strncpy(servName, &ibuf[index], nameLen);
		servName[nameLen-1]='\0';

		err = CheckService(servName, stat);		
		
		if(err!=0)
		{
			ok = false;
			switch(err)
			{
			case ERROR_INVALID_NAME:			
			case ERROR_SERVICE_DOES_NOT_EXIST:
				AddServStatToBuffer(obuf, servName, SERV_SERVICE_DOES_NOT_EXIST);
				break;
			case ERROR_SERVICE_NOT_ACTIVE:
				AddServStatToBuffer(obuf, servName, SERV_SERVICE_NOT_ACTIVE);
				break;
			case ERROR_SERVICE_REQUEST_TIMEOUT:
				AddServStatToBuffer(obuf, servName, SERV_SERVICE_REQUEST_TIMEOUT);
				break;
			default:
				AddServStatToBuffer(obuf, servName, SERV_CHECK_ERROR);							
			}
		}
		else
		{
			switch(stat)
			{
			case SERVICE_RUNNING:
			case SERVICE_START_PENDING:
				AddServStatToBuffer(obuf, servName, SERV_RUNNING);
				break;
			case SERVICE_PAUSED :
			case SERVICE_PAUSE_PENDING:
				AddServStatToBuffer(obuf, servName, SERV_PAUSED);
				ok=false;
				break;
			default:
				AddServStatToBuffer(obuf, servName, SERV_SERVICE_NOT_ACTIVE);				
				ok=false;
			}
		}

		delete servName;
	}
	
	strcat(obuf, ";");	

	bytesNeeded = strlen(obuf);
	char *tmp = new char[bytesNeeded+1];

	strcpy(tmp, obuf);
	
	sprintf( obuf, "nbytes=%d\0", bytesNeeded);

	int i=0;
	while(tmp[i]!=';')
		i++;
	
	strcat(obuf, &tmp[i]);	

	delete tmp;
	return 1;
}

VOID AddServStatToBuffer(char *buf, char* servName, int stat)
{
	char tmp[10];	

	strcat(buf, servName);
	strcat(buf, "=");
	strcat(buf, itoa(stat, tmp, 10));
	strcat(buf, ";");	
}  

DWORD GetBytesNeededForServiceCheckBuffer(char *ibuf)
{
	DWORD nBytes=0;
	DWORD index=0;
	DWORD i2=0;
	DWORD nameLen=0;
	DWORD stat;
	DWORD err;
	char *servName;
	
	while( strncmp(&ibuf[index], "key=", 4) != 0 && index < strlen(ibuf))
		index++;

	while(index < strlen(ibuf))
	{
		while( ibuf[index]!=';' && index < strlen(ibuf))
			index++;

		if(ibuf[index+1]==';')
			break;		

		i2 = ++index;

		while( ibuf[i2]!=';' && i2 < strlen(ibuf))
			i2++;

		nameLen = i2-index+1;

		servName = new char[nameLen];
		strncpy(servName, &ibuf[index], nameLen);
		servName[nameLen-1]='\0';		
				
		err = CheckService(servName, stat);		
			
		nBytes += strlen(servName) + 3;

		delete servName;
	}
	
	return nBytes;
}