/*******************************************************************
 *
 * NetSaintService for NT
 * Copyright (c) 2000 Jan Christian Kaldestad 
 *
 *
 * Description:
 * This service allows clients to check:
 *   1) processor load (average) and processes with highest cpu usage
 *   2) events in the eventlog
 *   3) status of a service
 *
 *
 ******************************************************************/

#include "nsserv.h"

// Global members
char		cliAddrStr[20];
int			stop=0;
char*		winDir;					// Windows directory.  The INIFILE has to reside in this directory !!
char		installDir[MAXPATH];	// Directory where we are installed
HANDLE		hCpuLoadThrd;
DWORD		dwCpuLoadThrdId;
BOOL		cpuThrdIsRunning;
HANDLE		hCheckEventLogThrd;
DWORD		dwCheckEventLogThrdId;
HANDLE		hNotifyEvenetLogThrd;
DWORD		dwNotifyEvenetLogThrdId;
int			eventLogCheckInterval;
HANDLE		hCheckEventLogEvent;
HANDLE		hMainContinueEvent;
struct ProcessInfo		Process;	// Info about process with highest cpu-load
struct CPUInfo			CPU;		// Info about the cpu
CRITICAL_SECTION		ProcessAccess;
int			servPort;
char		*passwd = new char[MAX_PASSWD_LEN];
HANDLE		hTCPThrd;
DWORD		dwTCPThrdId;


VOID ServiceStart(DWORD dwArgc, LPTSTR *lpszArgv) {	
	
	SOCKET		servSock;
	SA_IN		cliaddr;	
	int			len = sizeof(cliaddr);
	int			err, WSAerr;	
	char		ibuf[BUF_SIZE]; 
	char		obuf[BUF_SIZE];
	char		*msgBuf = new char[100];
	int			index=0;
		
	// initializing

	if( !Initialize(servSock, passwd) )
		return;
	
	// tell the SCM that we are started
	
	ReportStatusToSCMgr(SERVICE_RUNNING,NO_ERROR,30000);
	
	AddToMessageLog("NetSaint Service started", EVENTLOG_INFORMATION_TYPE);
	PrintToFile("---- STARTING ----", ERRLOG);

	for(int i=0; i<BUF_SIZE; i++)
		ibuf[i] = '\0';

	while(!stop) {		
		// receive client request		
		err = recvfrom(servSock, ibuf, sizeof(ibuf), 0, (sockaddr *) &cliaddr, &len);
		
		WSAerr = WSAGetLastError();		
		
		// if no requests available, wait and try again
		if(WSAerr == WSAEWOULDBLOCK ) {				
						
			// Check to see if threads are OK
			if( !cpuThrdIsRunning ) {
				PrintToFile("ERR:\tThe cpu-load thread has exited.  Service shuts down nowd", ERRLOG);
				AddToMessageLog("The cpu-load thread has exited.  Service shuts down now.", EVENTLOG_ERROR_TYPE);
			}			
			
			Sleep(500);
			continue;
		}		

		if(err == SOCKET_ERROR) {
			// if error in recvfrom, report it, wait and try again
			
			AddToMessageLog("Socket error in recvfrom", EVENTLOG_ERROR_TYPE);
			PrintSocketError(WSAerr, "recvfrom", ERRLOG);
			Sleep(500);
		}
		else {						
			strcpy(cliAddrStr, inet_ntoa((struct in_addr)cliaddr.sin_addr) );			
						
			err = AnalyzeUDPMessage(ibuf, obuf, passwd);			

			if(err != NOERR) {
				// if error in message, discard it and continue listening

				AddToMessageLog("Error in AnalyzeUDPMessage() in ServiceStart()", EVENTLOG_ERROR_TYPE);
				PrintToFile("ERR: AnalyzeUDPMessage() in ServiceStart()", ERRLOG);
				continue;
			}
						
			err = sendto(servSock, obuf, strlen(obuf),0, (sockaddr *) &cliaddr, len);
			WSAerr = WSAGetLastError();				

			if (err == SOCKET_ERROR) 
			{
				AddToMessageLog("Socket error in sendto", EVENTLOG_ERROR_TYPE);
				PrintSocketError(WSAerr, "sendto", ERRLOG);				
				continue;
			}			
		}			
	}
	
	ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);
	
	AddToMessageLog("NetSaint Service stopped", EVENTLOG_INFORMATION_TYPE);
	PrintToFile("---- STOPPED ----", ERRLOG);	
	
	closesocket(servSock);
	WSACleanup();	
}

int AnalyzeUDPMessage(char *ibuf, char *msgBuf, char *passwd) {
		
	int				err;
	int				id;	

	// get message id
	id = atoi(&ibuf[3]);

	// check password
	if(!CheckPasswd(ibuf, passwd))
		return ERR_AUTHENTICATION;	
	
	switch(id) 
	{
	case CPU_LOAD_QUERY:		
		err = GetCPULoadInfo(ibuf, msgBuf);		
		break;
	default:
		err = ERR_NO_SUCH_MESSAGE;		
	}	
	
	return err;
}

int GetCPULoadInfo(char *ibuf, char *obuf) {
	
	double load1, load2;

	DWORD	index = 0;
	int		interval1, interval2;

	while( strncmp(&ibuf[index], ";i1=", 4)!=0 && index < strlen(ibuf))
		index++;

	index+=4;

	interval1 = atoi(&ibuf[index])*60;

	while( strncmp(&ibuf[index], ";i2=", 4)!=0 && index < strlen(ibuf))
		index++;

	index+=4;

	interval2 = atoi(&ibuf[index])*60;
	
	if( (interval1 < 0) || (interval1 > CPULoadArraySize) 
		|| (interval2 < 0) || (interval2 > CPULoadArraySize))	
		return ERR_CPULOAD_INTERVAL;
		
	// get requested information
	EnterCriticalSection(&ProcessAccess);
		load1 = CPU.ComputeAvgCPULoad(interval1);		
		load2 = CPU.ComputeAvgCPULoad(interval2);		

		// message to send
		int j;
		j =  sprintf(obuf, "id=%d;", CPU_LOAD_QUERY);
		j += sprintf(obuf+j, "l1=%d;", Round(load1));
		j += sprintf(obuf+j, "l2=%d;", Round(load2));		
		j += sprintf(obuf+j, "pname=%s;", Process.processName);
		j += sprintf(obuf+j, "pload=%d;", Round(Process.cpuLoad));

		
	LeaveCriticalSection(&ProcessAccess);	
	
	return NOERR;
}


int Round(double num) {
	double tmp;
	tmp = num - floor(num);

	if(tmp < 0.5)
		return  floor(num);
	else
		return ( ceil(num));
}


int CreateUDPSocket(SOCKET &servSock, int port) {
	
	struct sockaddr_in servaddr;
	WORD		wVersionRequired;
	WSADATA		wsaData;
	int err, WSAerr;
	unsigned long socketio;
	
	wVersionRequired = MAKEWORD(1,1);	
	
	err = WSAStartup(wVersionRequired,&wsaData); 
	WSAerr = WSAGetLastError();
	
	if(err != 0) {		
		PrintSocketError(WSAerr, "CreateUDPSocket", ERRLOG);	
		return ERR_SOCKETINIT;	
	}	

	// create uninitialized UDP socket
	servSock = socket( AF_INET, SOCK_DGRAM, 0);	
	
	memset(&servaddr, 0, sizeof(servaddr));

	// initialize the server-adress	
	servaddr.sin_family		 = AF_INET; 
	servaddr.sin_addr.s_addr = INADDR_ANY; 
	servaddr.sin_port		 = htons(port); 
	
	
	err = bind(servSock, (sockaddr *) &servaddr, sizeof(servaddr));
	WSAerr = WSAGetLastError();
	

	if (err == SOCKET_ERROR) {		
		PrintSocketError(WSAerr, "CreateUDPSocket", ERRLOG);
		
		closesocket(servSock);
		WSACleanup();
		return ERR_BIND;		
	}

	// make the socket non-blocking
	socketio = 1;
	ioctlsocket(servSock, FIONBIO, &socketio);

	return NOERR;
}

int ReadInitData(char *passwd, int &port, char* installDir)
{
	HANDLE		hFile;
	int			err=0;
	int			forts=0;
	int			nLoop=0;
	char		_port[5];
	int			strSize=1000;
	char*		buf;
	DWORD		dwFileSize;

	char		fileLoc[256];
	
	fileLoc[0]='\0';
	strncat(fileLoc, winDir, 255-strlen(fileLoc));
	strncat(fileLoc, "\\", 255-strlen(fileLoc));
	strncat(fileLoc, INIFILE, 255-strlen(fileLoc));
		
	hFile = CreateFile( fileLoc, 
						GENERIC_READ,
						0,
						NULL,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL,
						NULL );
	
	if(hFile == INVALID_HANDLE_VALUE) {	
		return ERR_NOINIFILE;		
	}
	
	dwFileSize = GetFileSize(hFile, NULL);
	if(dwFileSize ==0xFFFFFFFF)
		return ERR_FILESIZE; // FileSize error	

	// allocate  buffer
	buf = new char[dwFileSize];

	do {
		ReadLine(buf, strSize, hFile);
		if( strcmp(buf,"key=") == 0 ) {				
			ReadLine(passwd, strSize, hFile);
			forts++;
		}
		else if( strcmp(buf,"port=") == 0) {
			ReadLine(_port, strSize, hFile);
			port = atoi(_port);
			forts++;
		}
		else if( strcmp(buf,"dir=") == 0) {		
			ReadLine(buf, strSize, hFile);			
			strncpy(installDir, buf, MAXPATH);			
			forts++;
		}

		nLoop++;				
	}	
	while(forts<3 && nLoop<1000); 
	
	
	delete buf;
	
	if(nLoop==1000) {		
		CloseHandle(hFile);
		return ERR_FILEFORMAT;		
	}

	CloseHandle(hFile);	
	return NOERR;
}

int ReadLine(char *str, int strSize, HANDLE hFile)
{
	int index=0, err;
	char ch=' ', forrige=' ';
	DWORD nRead;
	bool eof=false;
	bool br=false;	

	while(index < strSize) {
		forrige=ch;
		err= ReadFile( hFile,
					   &ch,
					   1,
			           &nRead,
				       NULL);

		str[index++]=ch;

		if( (err==1 && nRead==0) ) { // End of file
			str[index-1]='\0';
			return NOERR;;
		}

		if( ch=='\x0a' && forrige=='\x0d') {	// End of line
			str[index-2]='\0';
			return NOERR;;			
		}
		
		if( ch=='=' ) {
			str[index]='\0';			
			return NOERR;;
		}		
	}

	return 1; //ERROR
}


BOOL CheckThread(HANDLE hCpuLoadThrd, DWORD &dwCpuLoadThrdId) 
{	
	int err;

	if(!cpuThrdIsRunning) {	
		
		CloseHandle(hCpuLoadThrd);
		hCpuLoadThrd =	CreateThread(	NULL,
										0,
										CollectCpuLoad,
										NULL,
										0,
										&dwCpuLoadThrdId );
		err = GetLastError();

		// wait a little and check if the thread was successfully restarted
		
		if( !hCpuLoadThrd ) {
			int i;
			char errMsg[256];
			i  = sprintf(errMsg, "ERR:\tCreateThread in CheckThread\n");
			i += sprintf(errMsg + i, "\tSystem error message: %d\n", err);
			i += sprintf(errMsg + i, "\tFailed to restart the cpu-load thread");
			AddToMessageLog(errMsg, EVENTLOG_ERROR_TYPE);
			PrintToFile(errMsg, ERRLOG);
			return 0;
		}
		else {
			AddToMessageLog("cpu-thread was successfuly restarted", EVENTLOG_INFORMATION_TYPE);
			PrintToFile("INFO:\tcpu-thread was successfuly restarted", ERRLOG);
		}

	}

	return 1;
}

// called when the service is stopped
VOID ServiceStop() {
	
	ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);	
	
	// set the stop flag
	stop=1;
}

BOOL Initialize(SOCKET &servSock,char* passwd) {	

	int			err;
	UINT		winDirSize;
		
	cpuThrdIsRunning = true;

	// tell the SCM that we are starting up...

	ReportStatusToSCMgr(SERVICE_START_PENDING,NO_ERROR,30000);		
	
	// get required buffersize

	winDirSize = GetWindowsDirectory(winDir, 0);
	
	// allocate space for buffer
	
	winDir=0;
	winDir = new char[winDirSize+20];
	
	// get the Windows directory

	if(GetWindowsDirectory(winDir, winDirSize) == 0) {

		// operation failed, tell the SCM that we are stopping and exit
		
		ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);		
		PrintToFile("ERR: GetWindowsDirectory() in ServiceStart()", ERRLOG);
		PrintToFile("---- STOPPED ----", ERRLOG);
		return FALSE;
	}		
	
	// tell the SCM that we are still starting up...

	ReportStatusToSCMgr(SERVICE_START_PENDING,NO_ERROR,30000);

	// get values from INIFILE:
	//	- shared secret
	//	- portnumber
	//	- current directoy

	err = ReadInitData(passwd, servPort, installDir);	
	
	// add message-file to registry

	AddEventSource(installDir);	

	if( err != NOERR ) {
		
		// operation failed, tell the SCM that we are stopping and exit
		
		ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);
		PrintToFile("ERR: ReadInitData() in ServiceStart()", ERRLOG);
		PrintToFile("---- STOPPED ----", ERRLOG);
		return FALSE;	 	
	}
	
	// set current directory

	if( !SetCurrentDirectory(installDir) ) {
		
		// operation failed, tell the SCM that we are stopping and exit
		
		ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);
		PrintToFile("ERR: SetCurrentDirectory() in ServiceStart()", ERRLOG);
		PrintToFile("---- STOPPED ----", ERRLOG);
		return FALSE;	
	}
	
	
	// tell the SCM that we are still starting up...		

	ReportStatusToSCMgr(SERVICE_START_PENDING,NO_ERROR,30000);

	// create UDP socket

	err = CreateUDPSocket(servSock, servPort);
	
	if(err != NOERR) {
		
		// socket creation failed, tell the SCM and exit

		ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);
		
		PrintToFile("ERR: CreateUDPSocket() in ServiceStart()", ERRLOG);
		PrintToFile("---- STOPPED ----",ERRLOG);
		return FALSE;
	}
	
	// tell the SCM that we are still starting up...

	ReportStatusToSCMgr(SERVICE_START_PENDING,NO_ERROR,30000);

	// start the TCP-thread

	hTCPThrd = CreateThread( NULL,
							 0,
							 TCP_Connection,
							 NULL,
							 0,
							 &dwTCPThrdId );
	if( hTCPThrd == NULL ) {

		// thread creation failed, tell the SCM and exit

		ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);		
		PrintToFile("ERR: Create TCPThrd in ServiceStart()", ERRLOG);
		PrintToFile("---- STOPPED ----",ERRLOG);
		return FALSE;	
	}

	
	// initialize critical section before we start the CPU-load thread
	
	InitializeCriticalSection(&ProcessAccess);		
	
	// start the CPU-load thread

	hCpuLoadThrd = CreateThread( NULL,
								 0,
								 CollectCpuLoad,
								 NULL,
								 0,
								 &dwCpuLoadThrdId );	
	
	if( hCpuLoadThrd == NULL ) {

		// thread creation failed, tell the SCM and exit

		ReportStatusToSCMgr(SERVICE_STOP_PENDING,NO_ERROR,3000);		
		PrintToFile("ERR: Create CpuLoadThrd in ServiceStart()", ERRLOG);
		PrintToFile("---- STOPPED ----",ERRLOG);
		return FALSE;		
	}

	return TRUE;
}
